"""Asynchronous Python client for Elgato Lights."""
from pathlib import Path


def load_fixture(filename: str) -> str:
    """Load a fixture."""
    path = Path(__file__).parent / "fixtures" / filename
    return path.read_text()



"""Asynchronous Python client for Elgato Lights."""
from __future__ import annotations

import asyncio
import socket
from dataclasses import dataclass
from importlib import metadata
from typing import (
    TYPE_CHECKING,
    Any,
    Concatenate,
    ParamSpec,
    Self,
    TypedDict,
    TypeVar,
)

import orjson
from aiohttp.client import ClientError, ClientSession
from aiohttp.hdrs import METH_GET, METH_POST, METH_PUT
from yarl import URL

from .exceptions import ElgatoConnectionError, ElgatoError, ElgatoNoBatteryError
from .models import BatteryInfo, BatterySettings, Info, PowerOnBehavior, Settings, State

if TYPE_CHECKING:
    from collections.abc import Callable, Coroutine

_ElgatoT = TypeVar("_ElgatoT", bound="Elgato")
_R = TypeVar("_R")
_P = ParamSpec("_P")


def requires_battery(
    func: Callable[Concatenate[_ElgatoT, _P], Coroutine[Any, Any, _R]],
) -> Callable[Concatenate[_ElgatoT, _P], Coroutine[Any, Any, _R]]:
    """Decorate Elgato calls that require a device with a battery installed.

    A decorator that wraps and guards the passed in function, and checks if
    the device has a battery installed and only than calls the function.
    """

    async def handler(self: _ElgatoT, *args: _P.args, **kwargs: _P.kwargs) -> _R:
        """Handle calls to devices that require a battery."""
        if await self.has_battery() is False:
            raise ElgatoNoBatteryError
        return await func(self, *args, **kwargs)

    return handler


@dataclass
class Elgato:
    """Main class for handling connections with an Elgato Light."""

    host: str
    port: int = 9123
    request_timeout: int = 8
    session: ClientSession | None = None

    _close_session: bool = False
    _has_battery: bool | None = None

    async def _request(
        self,
        uri: str,
        *,
        method: str = METH_GET,
        data: dict[str, Any] | None = None,
    ) -> str:
        """Handle a request to a Elgato Light device.

        A generic method for sending/handling HTTP requests done against
        the Elgato Light API.

        Args:
        ----
            uri: Request URI, without '/elgato/', for example, 'info'
            method: HTTP Method to use.
            data: Dictionary of data to send to the Elgato Light.

        Returns:
        -------
            A Python string (JSON) with the response from the Elgato Light API.

        Raises:
        ------
            ElgatoConnectionError: An error occurred while communicating with
                the Elgato Light.
            ElgatoError: Received an unexpected response from the Elgato Light
                API.
        """
        version = metadata.version(__package__)
        url = URL.build(
            scheme="http",
            host=self.host,
            port=self.port,
            path="/elgato/",
        ).join(URL(uri))

        headers = {
            "User-Agent": f"PythonElgato/{version}",
            "Accept": "application/json, text/plain, */*",
        }

        if self.session is None:
            self.session = ClientSession()
            self._close_session = True

        try:
            async with asyncio.timeout(self.request_timeout):
                response = await self.session.request(
                    method,
                    url,
                    json=data,
                    headers=headers,
                )
                response.raise_for_status()
        except asyncio.TimeoutError as exception:
            msg = "Timeout occurred while connecting to Elgato Light device"
            raise ElgatoConnectionError(msg) from exception
        except (
            ClientError,
            socket.gaierror,
        ) as exception:
            msg = "Error occurred while communicating with Elgato Light device"
            raise ElgatoConnectionError(msg) from exception

        return await response.text()

    async def has_battery(self) -> bool:
        """Check if the Elgato Light device has a battery.

        Returns
        -------
            A boolean indicating if the Elgato Light device has a battery.
        """
        if self._has_battery is None:
            settings = await self.settings()
            self._has_battery = settings.battery is not None
        return self._has_battery

    @requires_battery
    async def battery(self) -> BatteryInfo:
        """Get battery information from Elgato Light device.

        Returns
        -------
            A BatteryInfo object, with information on the current battery state
            of the Elgato light.
        """
        data = await self._request("battery-info")
        return BatteryInfo.from_json(data)

    @requires_battery
    async def battery_bypass(self, *, on: bool) -> None:
        """Change the bypass mode of the Elgato Light device.

        In the app this is also called "Studio mode". When the bypass mode is
        the battery isn't used and would only work when the device is plugged
        into mains.

        There is an odd bug in current versions of the Elgato Light Mini
        firmware, that turns the light on when the bypass mode is turned off;
        the device will still think it is turned off, but the light will be on.

        Args:
        ----
            on: A boolean, true to turn on bypass, false otherwise.
        """
        await self._request(
            "/elgato/lights/settings",
            method=METH_PUT,
            data={"battery": {"bypass": int(on)}},
        )

    async def battery_settings(self) -> BatterySettings:
        """Get device battery settings from Elgato Light device.

        Guarded version of `settings().battery`.

        Returns
        -------
            A Battery settings object, with information about the Elgato Light device.
        """
        settings = await self.settings()
        if settings.battery is None:
            raise ElgatoNoBatteryError
        return settings.battery

    @requires_battery
    # pylint: disable-next=too-many-arguments
    async def energy_saving(  # noqa: PLR0913
        self,
        *,
        adjust_brightness: bool | None = None,
        brightness: int | None = None,
        disable_wifi: bool | None = None,
        minimum_battery_level: int | None = None,
        on: bool | None = None,
    ) -> None:
        """Change the energy saving mode of the Elgato Light device.

        Args:
        ----
            adjust_brightness: Adjust the brightness of the light when it drops
                below the minimum battery level threshold. True to turn it on,
                false otherwise.
            brightness: The brightness to set the light to when energy savings
                kicks in. This is only used when adjust_brightness is True.
            disable_wifi: Disable the WiFi of the Elgato Light device when
                energy savings kicks in. True to turn it on, false otherwise.
            minimum_battery_level: The minimum battery level threshold to
                trigger energy savings.
            on: A boolean, true to turn on energy saving, false otherwise.
        """
        current_settings = await self.battery_settings()
        data = current_settings.energy_saving.to_dict()

        if on is not None:
            data["enable"] = int(on)
        if minimum_battery_level is not None:
            data["minimumBatteryLevel"] = minimum_battery_level
        if disable_wifi is not None:
            data["disableWifi"] = int(disable_wifi)
        if adjust_brightness is not None:
            data["adjustBrightness"]["enable"] = int(adjust_brightness)
        if brightness is not None:
            data["adjustBrightness"]["brightness"] = brightness

        await self._request(
            "/elgato/lights/settings",
            method=METH_PUT,
            data={"battery": {"energySaving": data}},
        )

    async def info(self) -> Info:
        """Get devices information from Elgato Light device.

        Returns
        -------
            A Info object, with information about the Elgato Light device.
        """
        data = await self._request("accessory-info")
        return Info.from_json(data)

    async def settings(self) -> Settings:
        """Get device settings from Elgato Light device.

        Returns
        -------
            A Settings object, with information about the Elgato Light device.
        """
        data = await self._request("lights/settings")
        return Settings.from_json(data)

    async def state(self) -> State:
        """Get the current state of Elgato Light device.

        Returns
        -------
            A State object, with the current Elgato Light state.
        """
        data = await self._request("lights")
        # pylint: disable-next=no-member
        lights = orjson.loads(data)["lights"]
        return State.from_dict(lights[0])

    async def identify(self) -> None:
        """Identify this Elgato Light device by making it blink."""
        await self._request("identify", method=METH_POST)

    async def restart(self) -> None:
        """Restart the Elgato Light device."""
        await self._request("restart", method=METH_POST)

    async def display_name(self, name: str) -> None:
        """Change the display name of an Elgato Light device.

        Args:
        ----
            name: The name to give the Elgato Light device.
        """
        await self._request(
            "/elgato/accessory-info",
            method=METH_PUT,
            data={"displayName": name},
        )

    # pylint: disable-next=too-many-arguments
    async def light(  # noqa: PLR0913
        self,
        *,
        on: bool | None = None,
        brightness: int | None = None,
        hue: float | None = None,
        saturation: float | None = None,
        temperature: int | None = None,
    ) -> None:
        """Change state of an Elgato Light device.

        Args:
        ----
            on: A boolean, true to turn the light on, false otherwise.
            brightness: The brightness of the light, between 0 and 255.
            hue: The hue range as a float from 0 to 360 degrees.
            saturation: The color saturation as a float from 0 to 100.
            temperature: The color temperature of the light, in mired.

        Raises:
        ------
            ElgatoError: The provided values are invalid.
        """
        if temperature and (hue or saturation):
            msg = "Cannot set temperature together with hue or saturation"
            raise ElgatoError(msg)

        class LightState(TypedDict, total=False):
            """Describe state dictionary that can be set on a light."""

            brightness: int
            hue: float
            on: int
            saturation: float
            temperature: int

        state: LightState = {}

        if on is not None:
            state["on"] = int(on)

        if brightness is not None:
            if not 0 <= brightness <= 100:
                msg = "Brightness not between 0 and 100"
                raise ElgatoError(msg)
            state["brightness"] = brightness

        if hue is not None:
            if not 0 <= hue <= 360:
                msg = "Hue not between 0 and 360"
                raise ElgatoError(msg)
            state["hue"] = hue

        if saturation is not None:
            if not 0 <= saturation <= 100:
                msg = "Saturation not between 0 and 100"
                raise ElgatoError(msg)
            state["saturation"] = saturation

        if temperature is not None:
            if not 143 <= temperature <= 344:
                msg = "Color temperature out of range"
                raise ElgatoError(msg)
            state["temperature"] = temperature

        if not state:
            msg = "No parameters to set, light not adjusted"
            raise ElgatoError(msg)

        await self._request(
            "lights",
            method=METH_PUT,
            data={"numberOfLights": 1, "lights": [state]},
        )

    async def power_on_behavior(
        self,
        *,
        behavior: PowerOnBehavior | None = None,
        brightness: int | None = None,
        hue: float | None = None,
        temperature: int | None = None,
    ) -> None:
        """Change the power on behavior of the Elgato Light device.

        Args:
        ----
            behavior: The power on behavior to set.
            brightness: The power on brightness of the light, between 0 and 255.
            hue: The power on hue range as a float from 0 to 360 degrees.
            temperature: The power on color temperature of the light, in mired.
        """
        current_settings = await self.settings()
        if behavior is not None:
            current_settings.power_on_behavior = behavior
        if brightness is not None:
            current_settings.power_on_brightness = brightness
        if hue is not None:
            current_settings.power_on_hue = hue
        if temperature is not None:
            current_settings.power_on_temperature = temperature

        # Unset battery if present, needs special handling
        if current_settings.battery:
            current_settings.battery = None

        await self._request(
            "/elgato/lights/settings",
            method=METH_PUT,
            data=current_settings.to_dict(),
        )

    async def close(self) -> None:
        """Close open client session."""
        if self.session and self._close_session:
            await self.session.close()

    async def __aenter__(self) -> Self:
        """Async enter.

        Returns
        -------
            The Elgato object.
        """
        return self

    async def __aexit__(self, *_exc_info: object) -> None:
        """Async exit.

        Args:
        ----
            _exc_info: Exec type.
        """
        await self.close()



# pylint: disable=W0621
"""Asynchronous Python client for Elgato Lights."""

import asyncio

from elgato import Elgato, State


async def main() -> None:
    """Show example on controlling your Elgato Key device."""
    async with Elgato("elgato-key-light.local") as elgato:
        print(await elgato.info())

        print(await elgato.settings())

        state: State = await elgato.state()

        # Toggle the light
        await elgato.light(on=(not state.on))


if __name__ == "__main__":
    asyncio.run(main())



"""Asynchronous Python client for Elgato Lights."""
from .elgato import Elgato
from .exceptions import ElgatoConnectionError, ElgatoError, ElgatoNoBatteryError
from .models import (
    BatteryInfo,
    BatterySettings,
    BatteryStatus,
    EnergySavingAdjustBrightnessSettings,
    EnergySavingSettings,
    Info,
    PowerOnBehavior,
    PowerSource,
    Settings,
    State,
    Wifi,
)

__all__ = [
    "BatteryInfo",
    "BatterySettings",
    "BatteryStatus",
    "Elgato",
    "ElgatoConnectionError",
    "ElgatoError",
    "ElgatoNoBatteryError",
    "EnergySavingAdjustBrightnessSettings",
    "EnergySavingSettings",
    "Info",
    "PowerOnBehavior",
    "PowerSource",
    "Settings",
    "State",
    "Wifi",
]



"""Tests for restarting the Elgato Light device."""

from aiohttp import ClientSession
from aresponses import ResponsesMockServer

from elgato import Elgato


async def test_restart(aresponses: ResponsesMockServer) -> None:
    """Test restarting the Elgato Light."""
    aresponses.add(
        "example.com:9123",
        "/elgato/restart",
        "POST",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="",
        ),
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.restart()



"""Tests for retrieving information from the Elgato Light device."""
from __future__ import annotations

import pytest
from aiohttp import ClientResponse, ClientSession
from aresponses import Response, ResponsesMockServer

from elgato import Elgato, ElgatoError, State

from . import load_fixture


async def test_state_temperature(aresponses: ResponsesMockServer) -> None:
    """Test getting Elgato Light state in temperature mode."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("state-temperature.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        state: State = await elgato.state()
        assert state
        assert state.brightness == 21
        assert state.hue is None
        assert state.on
        assert state.saturation is None
        assert state.temperature == 297


async def test_state_color(aresponses: ResponsesMockServer) -> None:
    """Test getting Elgato Light state in color mode."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("state-color.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        state: State = await elgato.state()
        assert state
        assert state.brightness == 50
        assert state.hue == 358.0
        assert state.on
        assert state.saturation == 6.0
        assert state.temperature is None


async def test_change_state_temperature(aresponses: ResponsesMockServer) -> None:
    """Test changing Elgato Light State in temperature mode."""

    async def response_handler(request: ClientResponse) -> Response:
        """Response handler for this test."""
        data = await request.json()
        assert data == {
            "numberOfLights": 1,
            "lights": [{"on": 1, "brightness": 100, "temperature": 200}],
        }
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="{}",
        )

    aresponses.add("example.com:9123", "/elgato/lights", "PUT", response_handler)

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.light(on=True, brightness=100, temperature=200)


async def test_change_state_color(aresponses: ResponsesMockServer) -> None:
    """Test changing Elgato Light State in color mode."""

    async def response_handler(request: ClientResponse) -> Response:
        """Response handler for this test."""
        data = await request.json()
        assert data == {
            "numberOfLights": 1,
            "lights": [{"on": 1, "brightness": 100, "hue": 10.1, "saturation": 20.2}],
        }
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="{}",
        )

    aresponses.add("example.com:9123", "/elgato/lights", "PUT", response_handler)

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.light(on=True, brightness=100, hue=10.1, saturation=20.2)


@pytest.mark.parametrize(
    ("state", "message"),
    [
        (
            {"hue": 10.1, "temperature": 10},
            "Cannot set temperature together with hue or saturation",
        ),
        (
            {"saturation": 10.1, "temperature": 10},
            "Cannot set temperature together with hue or saturation",
        ),
        (
            {"brightness": -1},
            "Brightness not between 0 and 100",
        ),
        (
            {"brightness": 101},
            "Brightness not between 0 and 100",
        ),
        (
            {"hue": -1},
            "Hue not between 0 and 360",
        ),
        (
            {"hue": 360.1},
            "Hue not between 0 and 360",
        ),
        (
            {"saturation": -1},
            "Saturation not between 0 and 100",
        ),
        (
            {"saturation": 100.1},
            "Saturation not between 0 and 100",
        ),
        (
            {"temperature": 142},
            "Color temperature out of range",
        ),
        (
            {"temperature": 345},
            "Color temperature out of range",
        ),
        (
            {},
            "No parameters to set, light not adjusted",
        ),
    ],
)
async def test_change_state_errors(state: dict[str, int | float], message: str) -> None:
    """Test changing Elgato Light State with invalid values."""
    elgato = Elgato("example.com")
    with pytest.raises(ElgatoError, match=message):
        await elgato.light(**state)  # type: ignore[arg-type]



"""Tests for retrieving information from the Elgato Light device."""

import pytest
from aiohttp import ClientResponse, ClientSession
from aresponses import Response, ResponsesMockServer

from elgato import Elgato, ElgatoNoBatteryError, PowerOnBehavior, Settings

from . import load_fixture


async def test_settings_keylight(aresponses: ResponsesMockServer) -> None:
    """Test getting Elgato Light device settings."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-keylight.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        settings: Settings = await elgato.settings()
        assert settings
        assert settings.color_change_duration == 100
        assert settings.power_on_behavior == 1
        assert settings.power_on_brightness == 20
        assert settings.power_on_hue is None
        assert settings.power_on_saturation is None
        assert settings.power_on_temperature == 213
        assert settings.switch_off_duration == 300
        assert settings.switch_on_duration == 100
        assert settings.battery is None


async def test_settings_led_strip(aresponses: ResponsesMockServer) -> None:
    """Test getting Elgato Led Strip device settings."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-strip.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        settings: Settings = await elgato.settings()
        assert settings
        assert settings.color_change_duration == 150
        assert settings.power_on_behavior == 2
        assert settings.power_on_brightness == 40
        assert settings.power_on_hue == 40.0
        assert settings.power_on_saturation == 15.0
        assert settings.power_on_temperature == 0
        assert settings.switch_off_duration == 400
        assert settings.switch_on_duration == 150
        assert settings.battery is None


async def test_settings_key_light_mini(aresponses: ResponsesMockServer) -> None:
    """Test getting Elgato Light Mini device settings.

    This device has a battery
    """
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-key-light-mini.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        settings: Settings = await elgato.settings()
        assert settings
        assert settings.battery
        assert settings.battery.bypass is False
        assert settings.battery.energy_saving.disable_wifi is False
        assert settings.battery.energy_saving.enabled is False
        assert settings.battery.energy_saving.minimum_battery_level == 15
        assert settings.battery.energy_saving.adjust_brightness.brightness == 10
        assert settings.battery.energy_saving.adjust_brightness.enabled is False
        assert settings.color_change_duration == 100
        assert settings.power_on_behavior == 1
        assert settings.power_on_brightness == 20
        assert settings.power_on_hue is None
        assert settings.power_on_saturation is None
        assert settings.power_on_temperature == 230
        assert settings.switch_off_duration == 300
        assert settings.switch_on_duration == 100


async def test_battery_settings_keylight(aresponses: ResponsesMockServer) -> None:
    """Test getting Elgato Light battery settings."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-keylight.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        with pytest.raises(ElgatoNoBatteryError):
            await elgato.battery_settings()


async def test_battery_settings_key_light_mini(aresponses: ResponsesMockServer) -> None:
    """Test getting Elgato Light Mini device battery settings.

    This device has a battery
    """
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-key-light-mini.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        settings = await elgato.battery_settings()
        assert settings
        assert settings
        assert settings.bypass is False
        assert settings.energy_saving.disable_wifi is False
        assert settings.energy_saving.enabled is False
        assert settings.energy_saving.minimum_battery_level == 15
        assert settings.energy_saving.adjust_brightness.brightness == 10
        assert settings.energy_saving.adjust_brightness.enabled is False


async def test_energy_savings_no_battery(aresponses: ResponsesMockServer) -> None:
    """Test adjusting energy saving settings on a Elgato device without battery."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-keylight.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        assert await elgato.has_battery() is False
        with pytest.raises(
            ElgatoNoBatteryError,
            match="The Elgato light does not have a battery.",
        ):
            await elgato.energy_saving(on=True)


async def test_energy_savings_full(aresponses: ResponsesMockServer) -> None:
    """Test changing energy saving settings."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-key-light-mini.json"),
        ),
        repeat=2,
    )

    async def response_handler(request: ClientResponse) -> Response:
        """Response handler for this test."""
        data = await request.json()
        assert data == {
            "battery": {
                "energySaving": {
                    "adjustBrightness": {"brightness": 42, "enable": 1},
                    "disableWifi": 1,
                    "enable": 1,
                    "minimumBatteryLevel": 21,
                },
            },
        }
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="{}",
        )

    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "PUT",
        response_handler,
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.energy_saving(
            adjust_brightness=True,
            brightness=42,
            disable_wifi=True,
            minimum_battery_level=21,
            on=True,
        )


async def test_energy_savings_no_changes(aresponses: ResponsesMockServer) -> None:
    """Test changing energy saving settings."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-key-light-mini.json"),
        ),
        repeat=2,
    )

    async def response_handler(request: ClientResponse) -> Response:
        """Response handler for this test."""
        data = await request.json()
        assert data == {
            "battery": {
                "energySaving": {
                    "adjustBrightness": {"brightness": 10, "enable": False},
                    "disableWifi": False,
                    "enable": False,
                    "minimumBatteryLevel": 15,
                },
            },
        }
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="{}",
        )

    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "PUT",
        response_handler,
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.energy_saving()


async def test_power_on_behavior_full(aresponses: ResponsesMockServer) -> None:
    """Test changing power on behavior settings."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-keylight.json"),
        ),
        repeat=1,
    )

    async def response_handler(request: ClientResponse) -> Response:
        """Response handler for this test."""
        data = await request.json()
        assert data == {
            "colorChangeDurationMs": 100,
            "powerOnBehavior": 2,
            "powerOnBrightness": 42,
            "powerOnHue": 21.0,
            "powerOnTemperature": 242,
            "switchOffDurationMs": 300,
            "switchOnDurationMs": 100,
        }
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="{}",
        )

    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "PUT",
        response_handler,
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.power_on_behavior(
            behavior=PowerOnBehavior.USE_DEFAULTS,
            brightness=42,
            hue=21.0,
            temperature=242,
        )


async def test_power_on_behavior_no_changes(aresponses: ResponsesMockServer) -> None:
    """Test changing power on behavior settings."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-key-light-mini.json"),
        ),
        repeat=2,
    )

    async def response_handler(request: ClientResponse) -> Response:
        """Response handler for this test."""
        data = await request.json()
        assert data == {
            "colorChangeDurationMs": 100,
            "powerOnBehavior": 1,
            "powerOnBrightness": 20,
            "powerOnTemperature": 230,
            "switchOffDurationMs": 300,
            "switchOnDurationMs": 100,
        }
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="{}",
        )

    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "PUT",
        response_handler,
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.power_on_behavior()



"""Tests for the Elgato Lights Library."""
# pylint: disable=protected-access
import asyncio

import pytest
from aiohttp import ClientResponse, ClientSession
from aiohttp.hdrs import METH_POST, METH_PUT
from aresponses import Response, ResponsesMockServer

from elgato import Elgato
from elgato.exceptions import ElgatoConnectionError, ElgatoError


async def test_json_request(aresponses: ResponsesMockServer) -> None:
    """Test JSON response is handled correctly."""
    aresponses.add(
        "example.com:9123",
        "/elgato/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text='{"status": "ok"}',
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        response = await elgato._request("test")
        assert response == '{"status": "ok"}'
        await elgato.close()


async def test_internal_session(aresponses: ResponsesMockServer) -> None:
    """Test JSON response is handled correctly."""
    aresponses.add(
        "example.com:9123",
        "/elgato/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text='{"status": "ok"}',
        ),
    )
    async with Elgato("example.com") as elgato:
        response = await elgato._request("test")
        assert response == '{"status": "ok"}'


async def test_put_request(aresponses: ResponsesMockServer) -> None:
    """Test PUT requests are handled correctly."""
    aresponses.add(
        "example.com:9123",
        "/elgato/test",
        "PUT",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text='{"status": "ok"}',
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        response = await elgato._request("test", method=METH_PUT, data={})
        assert response == '{"status": "ok"}'


async def test_post_request(aresponses: ResponsesMockServer) -> None:
    """Test POST requests are handled correctly."""
    aresponses.add(
        "example.com:9123",
        "/elgato/test",
        "POST",
        Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text='{"status": "ok"}',
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        response = await elgato._request("test", method=METH_POST, data={})
        assert response == '{"status": "ok"}'


async def test_request_port(aresponses: ResponsesMockServer) -> None:
    """Test the Elgato Light running on non-standard port."""
    aresponses.add(
        "example.com:3333",
        "/elgato/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text='{"status": "ok"}',
        ),
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", port=3333, session=session)
        response = await elgato._request("test")
        assert response == '{"status": "ok"}'


async def test_timeout(aresponses: ResponsesMockServer) -> None:
    """Test request timeout from the Elgato Light."""

    # Faking a timeout by sleeping
    async def response_handler(_: ClientResponse) -> Response:
        """Response handler for this test."""
        await asyncio.sleep(2)
        return aresponses.Response(body="Goodmorning!")

    aresponses.add("example.com:9123", "/elgato/test", "GET", response_handler)

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session, request_timeout=1)
        with pytest.raises(ElgatoConnectionError):
            assert await elgato._request("test")


async def test_http_error400(aresponses: ResponsesMockServer) -> None:
    """Test HTTP 404 response handling."""
    aresponses.add(
        "example.com:9123",
        "/elgato/test",
        "GET",
        aresponses.Response(text="OMG PUPPIES!", status=404),
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        with pytest.raises(ElgatoError):
            assert await elgato._request("test")


async def test_light_on(aresponses: ResponsesMockServer) -> None:
    """Test controlling a Elgato Light."""

    # Handle to run asserts on request in
    async def response_handler(request: ClientResponse) -> Response:
        """Response handler for this test."""
        data = await request.json()
        assert data == {
            "numberOfLights": 1,
            "lights": [{"brightness": 100, "temperature": 275, "on": 1}],
        }
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text='{"status": "ok"}',
        )

    aresponses.add(
        "example.com:9123",
        "/elgato/lights",
        "PUT",
        response_handler,
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.light(on=True, brightness=100, temperature=275)


async def test_light_off(aresponses: ResponsesMockServer) -> None:
    """Test turning off an Elgato Light."""

    # Handle to run asserts on request in
    async def response_handler(request: ClientResponse) -> Response:
        """Response handler for this test."""
        data = await request.json()
        assert data == {
            "numberOfLights": 1,
            "lights": [{"on": 0}],
        }
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text='{"status": "ok"}',
        )

    aresponses.add(
        "example.com:9123",
        "/elgato/lights",
        "PUT",
        response_handler,
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.light(on=False)


async def test_light_no_on_off(aresponses: ResponsesMockServer) -> None:
    """Test controlling an Elgato Light without turning it on/off."""

    # Handle to run asserts on request in
    async def response_handler(request: ClientResponse) -> Response:
        """Response handler for this test."""
        data = await request.json()
        assert data == {
            "numberOfLights": 1,
            "lights": [{"brightness": 50}],
        }
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text='{"status": "ok"}',
        )

    aresponses.add(
        "example.com:9123",
        "/elgato/lights",
        "PUT",
        response_handler,
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.light(brightness=50)



# pylint: disable=W0621
"""Asynchronous Python client for Elgato Lights."""

import asyncio

from elgato import Elgato


async def main() -> None:
    """Show example of programmatically change the display name of a Elgato Light."""
    async with Elgato("elgato-key-light.local") as elgato:
        # Current name
        await elgato.info()

        # Change the name
        await elgato.display_name("New name")

        # New name
        await elgato.info()


if __name__ == "__main__":
    asyncio.run(main())



"""Tests for retrieving information from the Elgato Key Light device."""

from aiohttp import ClientResponse, ClientSession
from aresponses import Response, ResponsesMockServer

from elgato import Elgato, Info

from . import load_fixture


async def test_info_key_light(aresponses: ResponsesMockServer) -> None:
    """Test getting Elgato Key Light device information."""
    aresponses.add(
        "example.com:9123",
        "/elgato/accessory-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("info-key-light.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        info: Info = await elgato.info()
        assert info
        assert info.display_name == "Frenck"
        assert info.features == ["lights"]
        assert info.firmware_build_number == 218
        assert info.firmware_version == "1.0.3"
        assert info.hardware_board_type == 53
        assert info.mac_address == "AA:BB:CC:DD:EE:FF"
        assert info.product_name == "Elgato Key Light"
        assert info.serial_number == "CN11A1A00001"
        assert info.wifi
        assert info.wifi.frequency == 2400
        assert info.wifi.rssi == -48
        assert info.wifi.ssid == "Frenck-IoT"


async def test_info_key_light_air(aresponses: ResponsesMockServer) -> None:
    """Test getting Elgato Key Light Air device information."""
    aresponses.add(
        "example.com:9123",
        "/elgato/accessory-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("info-key-light-air.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        info: Info = await elgato.info()
        assert info
        assert not info.display_name
        assert info.features == ["lights"]
        assert info.firmware_build_number == 195
        assert info.firmware_version == "1.0.3"
        assert info.hardware_board_type == 200
        assert info.mac_address is None
        assert info.product_name == "Elgato Key Light Air"
        assert info.serial_number == "CW44J2A03032"
        assert info.wifi is None


async def test_info_light_strip(aresponses: ResponsesMockServer) -> None:
    """Test getting Elgato Light Strip device information."""
    aresponses.add(
        "example.com:9123",
        "/elgato/accessory-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("info-light-strip.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        info: Info = await elgato.info()
        assert info
        assert info.display_name == "Elgato Light"
        assert info.features == ["lights"]
        assert info.firmware_build_number == 211
        assert info.firmware_version == "1.0.4"
        assert info.hardware_board_type == 70
        assert info.mac_address is None
        assert info.product_name == "Elgato Light Strip"
        assert info.serial_number == "EW52J1A00082"
        assert info.wifi is None


async def test_change_display_name(aresponses: ResponsesMockServer) -> None:
    """Test changing the display name of an Elgato Light."""

    async def response_handler(request: ClientResponse) -> Response:
        """Response handler for this test."""
        data = await request.json()
        assert data == {"displayName": "OMG PUPPIES"}
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="",
        )

    aresponses.add(
        "example.com:9123",
        "/elgato/accessory-info",
        "PUT",
        response_handler,
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.display_name("OMG PUPPIES")


async def test_missing_display_name(aresponses: ResponsesMockServer) -> None:
    """Test ensure we can handle a missing display name."""
    aresponses.add(
        "example.com:9123",
        "/elgato/accessory-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("info-light-strip.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        info: Info = await elgato.info()
        assert info
        assert info.display_name == "Elgato Light"



"""Exceptions for Elgato Lights."""

from typing import Any


class ElgatoError(Exception):
    """Generic Elgato Light exception."""


class ElgatoConnectionError(ElgatoError):
    """Elgato Light connection exception."""


class ElgatoNoBatteryError(Exception):
    """Elgato light does not have a battery."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize the ElgatoNoBatteryError."""
        if not args:  # pragma: no cover
            args = ("The Elgato light does not have a battery.",)
        super().__init__(*args, **kwargs)



# pylint: disable=W0621
"""Asynchronous Python client for Elgato Lights."""

import asyncio

from elgato import Elgato


async def main() -> None:
    """Show example on how to work with an battery powered device."""
    async with Elgato("10.10.11.172") as elgato:
        # General device information
        print(await elgato.info())
        print(settings := await elgato.settings())
        print(await elgato.state())

        # General battery information
        battery = await elgato.battery()
        print(f"Level: {battery.level} %")
        print(f"Power: {battery.charge_power}W")
        print(f"Voltage: {battery.charge_voltage}V")
        print(f"Current: {battery.charge_current}A")

        # Toggle the studio mode
        if settings.battery is None:
            raise RuntimeError
        await elgato.battery_bypass(on=(not settings.battery.bypass))

        # Adjust energy saving settings
        await elgato.energy_saving(
            on=True,
            brightness=66,
            disable_wifi=True,
            minimum_battery_level=16,
            adjust_brightness=True,
        )


if __name__ == "__main__":
    asyncio.run(main())



"""Tests for retrieving battery information from the Elgato Key Light device."""

import pytest
from aiohttp import ClientResponse, ClientSession
from aresponses import Response, ResponsesMockServer

from elgato import BatteryInfo, BatteryStatus, Elgato, ElgatoNoBatteryError, PowerSource

from . import load_fixture


async def test_battery_info_no_battery(aresponses: ResponsesMockServer) -> None:
    """Test getting battery information from a Elgato device without battery."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-keylight.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        assert await elgato.has_battery() is False
        with pytest.raises(
            ElgatoNoBatteryError,
            match="The Elgato light does not have a battery.",
        ):
            await elgato.battery()


async def test_battery_info(aresponses: ResponsesMockServer) -> None:
    """Test getting battery information from Elgato Key Light Mini device."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-key-light-mini.json"),
        ),
    )
    aresponses.add(
        "example.com:9123",
        "/elgato/battery-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("battery-info.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        battery: BatteryInfo = await elgato.battery()
        assert battery
        assert battery.charge_current == 3.01
        assert battery.charge_power == 12.66
        assert battery.charge_voltage == 4.21
        assert battery.input_charge_current == 3008
        assert battery.input_charge_power == 12658
        assert battery.input_charge_voltage == 4208
        assert battery.level == 78.57
        assert battery.power_source == PowerSource.MAINS
        assert battery.status == BatteryStatus.CHARGING


async def test_battery_bypass_no_battery(aresponses: ResponsesMockServer) -> None:
    """Test enabling battery bypass on a Elgato device without battery."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-keylight.json"),
        ),
    )
    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        assert await elgato.has_battery() is False
        with pytest.raises(
            ElgatoNoBatteryError,
            match="The Elgato light does not have a battery.",
        ):
            await elgato.battery_bypass(on=True)


async def test_battery_bypass(aresponses: ResponsesMockServer) -> None:
    """Test changing battery bypass / studio mode."""
    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("settings-key-light-mini.json"),
        ),
    )

    async def response_handler(request: ClientResponse) -> Response:
        """Response handler for this test."""
        data = await request.json()
        assert data == {"battery": {"bypass": 1}}
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="{}",
        )

    aresponses.add(
        "example.com:9123",
        "/elgato/lights/settings",
        "PUT",
        response_handler,
    )

    async with ClientSession() as session:
        elgato = Elgato("example.com", session=session)
        await elgato.battery_bypass(on=True)



"""Asynchronous Python client for Elgato Lights."""
from __future__ import annotations

from dataclasses import dataclass, field
from enum import IntEnum

from mashumaro import field_options
from mashumaro.config import BaseConfig
from mashumaro.mixins.orjson import DataClassORJSONMixin
from mashumaro.types import SerializationStrategy


class IntegerIsBoolean(SerializationStrategy):
    """Boolean serialization strategy for integers."""

    def serialize(self, value: bool) -> int:  # noqa: FBT001
        """Serialize a boolean to an integer."""
        return int(value)

    def deserialize(self, value: int) -> bool:
        """Deserialize an integer to a boolean."""
        return bool(value)


class BaseModel(DataClassORJSONMixin):
    """Base model for all Elgato models."""

    # pylint: disable-next=too-few-public-methods
    class Config(BaseConfig):
        """Mashumaro configuration."""

        omit_none = True
        serialization_strategy = {bool: IntegerIsBoolean()}  # noqa: RUF012
        serialize_by_alias = True


@dataclass
class EnergySavingAdjustBrightnessSettings(BaseModel):
    """Object holding the Elgato Light battery energy saving settings for brightness.

    This object holds information about the Elgato Light energy saving settings
    for saving battery when using the light regarding brightness limits.

    Only applies to Elgato devices with a battery of course,
    like the Key Light Mini.

    Attributes
    ----------
        brightness: Adjusted brightness when energy saving is active.
        enabled: boolean
    """

    brightness: int
    enabled: bool = field(metadata=field_options(alias="enable"))


@dataclass
class EnergySavingSettings(BaseModel):
    """Object holding the Elgato Light battery energy saving settings.

    This object holds information about the Elgato Light energy saving settings
    for saving battery when using the light.

    Only applies to Elgato devices with a battery of course,
    like the Key Light Mini.

    Attributes
    ----------
        adjust_brightness: Adjust brightness when energy saving is active.
        disable_wifi: Disable Wi-Fi when energy saving is active.
        enabled: boolean
        minimum_battery_level: Use energy saving when battery level is below this value.
    """

    adjust_brightness: EnergySavingAdjustBrightnessSettings = field(
        metadata=field_options(alias="adjustBrightness")
    )
    disable_wifi: bool = field(metadata=field_options(alias="disableWifi"))
    enabled: bool = field(metadata=field_options(alias="enable"))
    minimum_battery_level: int = field(
        metadata=field_options(alias="minimumBatteryLevel")
    )


@dataclass
class BatterySettings(BaseModel):
    """Object holding the Elgato Light battery information.

    This object holds information about the Elgato Light battery.

    Only applies to Elgato devices with a battery of course,
    like the Key Light Mini.

    Attributes
    ----------
        energy_saving: Energy saving settings.
        bypass: If the battery is bypassed (studio mode).
    """

    energy_saving: EnergySavingSettings = field(
        metadata=field_options(alias="energySaving")
    )
    bypass: bool


@dataclass
class Wifi(BaseModel):
    """Object holding the Elgato device Wi-Fi information.

    This object holds wireles information about the Elgato device.

    Attributes
    ----------
        frequency: The frequency in MHz of the Wi-Fi network connected.
        rssi: The signal strength in dBm of the Wi-Fi network connected.
        ssid: The SSID of the Wi-Fi network the device is connected to.
    """

    frequency: int = field(metadata=field_options(alias="frequencyMHz"))
    rssi: int
    ssid: str


class PowerSource(IntEnum):
    """Enum for the power source of the Elgato Light."""

    UNKNOWN = 0
    MAINS = 1
    BATTERY = 2


class BatteryStatus(IntEnum):
    """Enum for the battery status of the Elgato Light.

    Value "1" seems to be unused. I could not get it to show up, no
    matter if the device was charging or not, in saving mode or even bypass.
    """

    DRAINING = 0
    CHARGING = 2
    CHARGED = 3


@dataclass
class BatteryInfo(BaseModel):
    """Object holding the Elgato Light device information.

    This object holds information about the Elgato Light.

    Attributes
    ----------
        charge_current: The charge current in A.
        charge_power: The charge power in W.
        charge_voltage: The charge voltage in V.
        input_charge_current: The charge current in mA.
        input_charge_voltage: The charge voltage in mV.
        input_charge_voltage: The charge voltage in mV.
        level: The battery level of the device in %.
        power_source: The power source of the device.
        status: The battery status.
        voltage: The current battery voltage in mV.
    """

    power_source: PowerSource = field(metadata=field_options(alias="powerSource"))
    level: float
    status: BatteryStatus
    voltage: int = field(metadata=field_options(alias="currentBatteryVoltage"))
    input_charge_voltage: int = field(
        metadata=field_options(alias="inputChargeVoltage")
    )
    input_charge_current: int = field(
        metadata=field_options(alias="inputChargeCurrent")
    )

    @property
    def input_charge_power(self) -> int:
        """Return the input charge power in mW."""
        return round(self.input_charge_voltage * self.input_charge_current / 1_000)

    @property
    def charge_current(self) -> float:
        """Return the charge current in A."""
        return round(self.input_charge_current / 1_000, 2)

    @property
    def charge_power(self) -> float:
        """Return the charge power in W."""
        return round(self.input_charge_power / 1_000, 2)

    @property
    def charge_voltage(self) -> float:
        """Return the charge voltage in V."""
        return round(self.input_charge_voltage / 1_000, 2)


@dataclass
# pylint: disable-next=too-many-instance-attributes
class Info(BaseModel):
    """Object holding the Elgato Light device information.

    This object holds information about the Elgato Light.

    Attributes
    ----------
        display_name: Configured display name of the Elgato Light.
        features: List of features this devices exposes.
        firmware_build_number: An integer with the build number of the firmware.
        firmware_version: String containing the firmware version.
        hardware_board_type: An integer indicating the board revision.
        product_name: The product name.
        serial_number: Serial number of the Elgato Light.
    """

    features: list[str]
    firmware_build_number: int = field(
        metadata=field_options(alias="firmwareBuildNumber")
    )
    firmware_version: str = field(metadata=field_options(alias="firmwareVersion"))
    hardware_board_type: int = field(metadata=field_options(alias="hardwareBoardType"))
    product_name: str = field(metadata=field_options(alias="productName"))
    serial_number: str = field(metadata=field_options(alias="serialNumber"))
    display_name: str = field(
        default="Elgato Light", metadata=field_options(alias="displayName")
    )
    mac_address: str | None = field(
        default=None, metadata=field_options(alias="macAddress")
    )
    wifi: Wifi | None = field(default=None, metadata=field_options(alias="wifi-info"))


class PowerOnBehavior(IntEnum):
    """Enum for the power on behavior of the Elgato Light."""

    UNKNOWN = 0
    RESTORE_LAST = 1
    USE_DEFAULTS = 2


@dataclass
# pylint: disable-next=too-many-instance-attributes
class Settings(BaseModel):
    """Object holding the Elgato Light device settings.

    This object holds information about the Elgato Light settings.

    Attributes
    ----------
        battery: Battery settings, if the device has a battery.
        color_change_duration: Transition time of color changes in milliseconds.
        power_on_behavior: 1 = Restore last, 2 = Use defaults.
        power_on_brightness: The brightness used as default.
        power_on_hue: The hue value used as default.
        power_on_saturation: The saturation level used as default.
        power_on_temperature: The temperature level used as default.
        switch_off_duration: Turn off transition time in milliseconds.
        switch_on_duration: Turn on transition time in milliseconds.
    """

    color_change_duration: int = field(
        metadata=field_options(alias="colorChangeDurationMs")
    )
    power_on_behavior: PowerOnBehavior = field(
        metadata=field_options(alias="powerOnBehavior")
    )
    power_on_brightness: int = field(metadata=field_options(alias="powerOnBrightness"))
    switch_off_duration: int = field(
        metadata=field_options(alias="switchOffDurationMs")
    )
    switch_on_duration: int = field(metadata=field_options(alias="switchOnDurationMs"))
    battery: BatterySettings | None = None
    power_on_hue: float | None = field(
        default=None, metadata=field_options(alias="powerOnHue")
    )
    power_on_saturation: float | None = field(
        default=None, metadata=field_options(alias="powerOnSaturation")
    )
    power_on_temperature: int | None = field(
        default=None, metadata=field_options(alias="powerOnTemperature")
    )


@dataclass
class State(BaseModel):
    """Object holding the Elgato Light state.

    Represents a visible state of an Elgato Light.

    Attributes
    ----------
        on: A boolean indicating the if the light if on or off.
        brightness: An integer between 0 and 255, representing the brightness.
        hue:
        saturation:
        temperature: An integer representing the color temperature in mireds.
    """

    on: bool
    brightness: int
    hue: float | None = field(default=None)
    saturation: float | None = field(default=None)
    temperature: int | None = field(default=None)
